#ifndef HARKIO_HARK_HARKFILEBINARYWRITER_HPP_
#define HARKIO_HARK_HARKFILEBINARYWRITER_HPP_

#include <ostream>
#include <sstream>
#include <boost/shared_ptr.hpp>
#include <boost/scoped_ptr.hpp>
#include <harkio2/InvalidHarkFile.hpp>
#include <harkio2/hark/HarkFileWriter.hpp>

/** @file HarkFileBinaryWriter.hpp
 * @brief HarkFileBinaryWriter class definition.
 */

namespace harkio2
{
namespace hark
{

class HarkFile;

/** @brief Binary file writer for HarkFile.
 */
class HarkFileBinaryWriter : public HarkFileWriter
{
public:
    /** @brief Constructs binary writer.
     * Data destination is std::ostream specified by 'out'.
     * The output stream doesn't have to support seeking.
     * The output stream must have been opend with binary mode.
     * @param out The output stream to which this instance writes data.
     */
    HarkFileBinaryWriter(std::ostream& out);

    /** @brief Virtual destructor.
     * This destructor must be declared as non-inline function
     * so that boost::scoped_ptr can delete its content.
     */
    virtual ~HarkFileBinaryWriter();

    /** @brief Writes HarkFile data structure to binary file.
     * @param hark The data to write.
     */
    virtual void Write(boost::shared_ptr<harkio2::hark::HarkFile> hark) throw (harkio2::InvalidHarkFile);

    virtual void Write(const harkio2::hark::HarkFile& o) throw (harkio2::InvalidHarkFile);
    virtual void Write(const harkio2::hark::hgtf::Fmt1_0& o) throw (harkio2::InvalidHarkFile);
    virtual void Write(const harkio2::hark::hgtf::Spec1_0& o) throw (harkio2::InvalidHarkFile);
    virtual void Write(const harkio2::hark::hgtf::M2PG1_0& o) throw (harkio2::InvalidHarkFile);
    virtual void Write(const harkio2::hark::hgtf::GTF1_0& o) throw (harkio2::InvalidHarkFile);
    virtual void Write(const harkio2::hark::hgtf::SM1_0& o) throw (harkio2::InvalidHarkFile);

#ifdef HARKIO_UNITTEST
public:
#else
private:
#endif
    /// @brief Helper class.
    class Helper;
    boost::scoped_ptr<Helper> helper;
private:
    /// @brief Data destination.
    std::ostream* out;

    /// @brief A temporary buffer.
    std::ostringstream temp_oss;
};

}
}

#endif // HARKIO_HARK_HARKFILEBINARYWRITER_HPP_

