#ifndef HARKIO_HARKCONFIG_MICARRAYLOCATIONFILE_HPP_
#define HARKIO_HARKCONFIG_MICARRAYLOCATIONFILE_HPP_

#include <string>
#include <vector>
#include <glibmm/ustring.h>
#include <harkio2/Version.hpp>
#include <harkio2/defineitem.hpp>
#include <harkio2/harkconfig/HarkConfigFile.hpp>
#include <harkio2/harkconfig/Mic.hpp>
#include <harkio2/harkconfig/HarkConfigFileWriter.hpp>

/** @file MicArrayLocationFile.hpp
 * @brief MicArrayLocationFile class and subclasses definitions.
 */

namespace harkio2
{
namespace harkconfig
{

/** @brief Abstract Mic Array Location file.
 * This class represents a file which contains a array of mic locations.
 * The file can be any version.
 */
class MicArrayLocationFile : public HarkConfigFile
{
public:
    /** @brief Returns the file type identifier.
     * @return The file type identifier which this class represents.
     */
    virtual std::string GetId() const { return "MicArrayLocationFile"; }
};

/** @brief Mic Array Location file version 2.0.
 */
class MicArrayLocationFile2_0 : public MicArrayLocationFile
{
public:
    /** @brief Returns the file version.
     * @return The version which this class represents.
     */
    virtual harkio2::Version GetVersion() const { return Version(2, 0); }

    DEFINE_ITEM(comment, Glib::ustring, Comment, const Glib::ustring&)
    DEFINE_ITEM(mics, std::vector<harkio2::harkconfig::Mic1>, Mics, const std::vector<harkio2::harkconfig::Mic1>&)

    /// @brief Implementation of Writable interface.
    void Write(HarkConfigFileWriter& writer) const throw (harkio2::InvalidHarkFile)
    {
        writer.Write(*this);
    }

};

}
}

#endif // HARKIO_HARKCONFIG_MICARRAYLOCATIONFILE_HPP_

