#include <harkio2/harkconfig/ImpulseResponseListFileTextHelper.hpp>

#include <vector>

#include <boost/regex.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/format.hpp>

#include <harkio2/Version.hpp>
#include <harkio2/RegexPatterns.hpp>
#include <harkio2/harkconfig/HarkConfigFile.hpp>
#include <harkio2/harkconfig/ImpulseResponseListFile.hpp>
#include <harkio2/harkconfig/Source.hpp>

using namespace std;
using namespace boost;

namespace RegexPatterns = harkio2::RegexPatterns;

namespace
{
/* common regex instances
 */

/// @brief A regex for "Version" line.
const static boost::regex regexVersion(
        (format("Version\\s*=?\\s*(%1%)\\.(%1%)") % RegexPatterns::INTEGER).str());

/// @brief A regex for "Source" line.
const static boost::regex regexSource(
        (format("(%1%)\\s*,\\s*(%1%)\\s*,\\s*(%1%)\\s*%%%%(.+),\\s*(%2%)") % RegexPatterns::DECIMAL % RegexPatterns::INTEGER).str());

}

namespace harkio2
{
namespace harkconfig
{

boost::shared_ptr<HarkConfigFile> ImpulseResponseListFileTextHelper::GetData()
{
    return versionHelper ? versionHelper->GetData() : boost::shared_ptr<HarkConfigFile>();
}

void ImpulseResponseListFileTextHelper::AnalyzeLine(const std::string& line, unsigned int lineNum)
{
    if (versionHelper)
    {
        // If the data structure's version is already revealed,
        // delegates to helper.
        versionHelper->AnalyzeLine(line, lineNum);
    }
    else
    {
        // Otherwise; waits a line, "Version = integer.integer"
        smatch m;

        if (regex_match(line, m, regexVersion))
        {
            Version v(lexical_cast<unsigned int>(m.str(1)), lexical_cast<unsigned int>(m.str(2)));
            if (v == Version(1, 0))
            {
                versionHelper.reset(new ImpulseResponseListFile1_0TextHelper());
            }
            else
            {
                throw AnalyzeError((format("unsupported version %s - ImpulseResponseListFile") % v.ToString()).str(), lineNum);
            }
        }
    }

}

void ImpulseResponseListFileTextHelper::Write(std::ostream& out)
{
}

ImpulseResponseListFile1_0TextHelper::ImpulseResponseListFile1_0TextHelper() :
    data(new ImpulseResponseListFile1_0()),
    sources(new std::vector<Source1>())
{
}

boost::shared_ptr<HarkConfigFile> ImpulseResponseListFile1_0TextHelper::GetData()
{
    if (data)
    {
        data->SetSources(*sources);
    }
    return static_pointer_cast<HarkConfigFile>(data);
}

void ImpulseResponseListFile1_0TextHelper::AnalyzeLine(const std::string& line, unsigned int lineNum)
{
    smatch m;

    if (regex_match(line, m, regexSource))
    {
        Source1 s;
        try
        {
            s.SetPos(Vector3<float>(
                        lexical_cast<float>(m.str(1)),
                        lexical_cast<float>(m.str(2)),
                        lexical_cast<float>(m.str(3))));
        }
        catch (const bad_lexical_cast&)
        {
            // This exception may never be thrown
            // because regex_match makes sure that this line is well formed.
            throw lineNum > 0 ? AnalyzeError("invalid position", lineNum) : AnalyzeError("invalid position");
        }
        s.SetFile(m.str(4));
        try
        {
            s.SetCutStart(lexical_cast<unsigned int>(m.str(5)));
        }
        catch (const bad_lexical_cast&)
        {
            // This exception may never be thrown
            // because regex_match makes sure that this line is well formed.
            throw lineNum > 0 ? AnalyzeError("invalid cut start", lineNum) : AnalyzeError("invalid cut start");
        }
        sources->push_back(s);
    }
    else
    {
        throw lineNum > 0 ? AnalyzeError("invalid line", lineNum) : AnalyzeError("invalid line");
    }
}

void ImpulseResponseListFile1_0TextHelper::Write(std::ostream& out)
{
}

}
}

