#define BOOST_TEST_MODULE "testcase for HarkFileBinaryWriter::Helper class"
#define BOOST_TEST_DYN_LINK
#include "common.hpp"

#include <boost/test/unit_test.hpp>

#define HARKIO_UNITTEST

#include <vector>
#include <sstream>
#include <harkio2/hark/HarkFileWriter.hpp>
#include <harkio2/hark/HarkFileReader.hpp>

#include "../src/hark/HarkFileBinaryWriter/Helper.cpp"
#include "../src/hark/HarkFileBinaryReader/Helper.cpp"

using std::vector;
using std::complex;

#if 0
BOOST_AUTO_TEST_CASE(calcchunksize_test)
{
    std::stringstream out;

    harkio2::hark::HarkFileBinaryWriter writer(out);
    harkio2::hark::HarkFileBinaryWriter::Helper helper(writer);

    vector<harkio2::Vector3<float>> micpos_fmt(10);
    boost::shared_ptr<harkio2::hark::hgtf::Fmt1_0> fmt10(
            new harkio2::hark::hgtf::Fmt1_0());
    fmt10->SetNMic(10);
    fmt10->SetMicPositions(micpos_fmt);
    BOOST_CHECK_EQUAL(helper.CalcChunkSizeProperty(*fmt10), 24 + 12 * fmt10->GetMicPositions().size());


    boost::shared_ptr<harkio2::hark::hgtf::Spec1_0> spec10(
            new harkio2::hark::hgtf::Spec1_0());
    BOOST_CHECK_EQUAL(helper.CalcChunkSizeProperty(*spec10), 1551);

    vector<vector<vector<vector<complex<float>>>>> tfs_m2pg(
            1, vector<vector<vector<complex<float>>>>(
                2, vector<vector<complex<float>>>(
                    3, vector<complex<float>>(
                        4))));
    boost::shared_ptr<harkio2::hark::hgtf::M2PG1_0> m2pg10(
            new harkio2::hark::hgtf::M2PG1_0());
    m2pg10->SetND(1);
    m2pg10->SetNR(2);
    m2pg10->SetTFs(tfs_m2pg);
    BOOST_CHECK_EQUAL(helper.CalcChunkSizeProperty(*m2pg10), 12 + 12 * 1 * 2 + 8 * 1 * 2 * 3 * 4);

    vector<vector<vector<complex<float>>>> tfs_gtf(
            10, vector<vector<complex<float>>>(
                20, vector<complex<float>>(
                    30)));
    boost::shared_ptr<harkio2::hark::hgtf::GTF1_0> gtf10(
            new harkio2::hark::hgtf::GTF1_0());
    gtf10->SetNSource(10);
    gtf10->SetNFIndex(30);
    gtf10->SetTFs(tfs_gtf);
    BOOST_CHECK_EQUAL(helper.CalcChunkSizeProperty(*gtf10), 8 + 12 * 10 + 4 + 4 * 30 + 8 * 10 * 20 * 30);

    vector<vector<vector<complex<float>>>> tfs_sm(
            20, vector<vector<complex<float>>>(
                10, vector<complex<float>>(
                    30)));
    boost::shared_ptr<harkio2::hark::hgtf::SM1_0> sm10(
            new harkio2::hark::hgtf::SM1_0());
    sm10->SetNSource(20);
    sm10->SetNFIndex(30);
    sm10->SetSMs(tfs_sm);
    BOOST_CHECK_EQUAL(helper.CalcChunkSizeProperty(*sm10), 8 + 12 * 20 + 4 + 4 * 30 + 8 * 20 * 10 * 30);

}
#endif


BOOST_AUTO_TEST_CASE(fmt10_test)
{
    std::stringstream out;

    harkio2::hark::HarkFileBinaryWriter writer(out);
    harkio2::hark::HarkFileBinaryWriter::Helper helper(writer);

    vector<harkio2::Vector3<float>> micpos_fmt(4, harkio2::Vector3<float>(1, 2, 3));
    boost::shared_ptr<harkio2::hark::hgtf::Fmt1_0> fmt10(
            new harkio2::hark::hgtf::Fmt1_0());
    fmt10->SetFormatId(1);
    fmt10->SetNMic(4);
    fmt10->SetNFFT(8);
    fmt10->SetNImpulseResponse(16);
    fmt10->SetFS(16000);
    fmt10->SetMicPositions(micpos_fmt);
    helper.Write(*fmt10);

    harkio2::hark::HarkFileBinaryReader reader(out);
    harkio2::hark::HarkFileBinaryReader::Helper rhelper(reader);

    auto rfmt10 = rhelper.ReadFmt1_0();
    BOOST_REQUIRE(rfmt10);
    BOOST_CHECK_EQUAL(rfmt10->GetFormatId(), 1);
    BOOST_CHECK_EQUAL(rfmt10->GetFS(), 16000);
    BOOST_CHECK_EQUAL(rfmt10->GetMicPositions().size(), micpos_fmt.size());
}

BOOST_AUTO_TEST_CASE(spec10_test)
{
    std::stringstream out;

    harkio2::hark::HarkFileBinaryWriter writer(out);
    harkio2::hark::HarkFileBinaryWriter::Helper helper(writer);

    boost::shared_ptr<harkio2::hark::hgtf::Spec1_0> spec10(
            new harkio2::hark::hgtf::Spec1_0());
    spec10->SetDate(u8"2011/08/02");
    spec10->SetLocation(u8"honda");
    spec10->SetRobot(u8"android");
    spec10->SetMicArrayType(u8"twitter");
    spec10->SetCreator(u8"nakadai");
    spec10->SetMemo(u8"アシモ");
    helper.Write(*spec10);

    harkio2::hark::HarkFileBinaryReader reader(out);
    harkio2::hark::HarkFileBinaryReader::Helper rhelper(reader);

    auto rspec10 = rhelper.ReadSpec1_0();
    BOOST_REQUIRE(rspec10);
    BOOST_CHECK_EQUAL(rspec10->GetDate(), u8"2011/08/02");
    BOOST_CHECK_EQUAL(rspec10->GetMemo(), u8"アシモ");
}

BOOST_AUTO_TEST_CASE(m2pg10_test)
{
    typedef std::complex<float> C;

    std::stringstream out;

    harkio2::hark::HarkFileBinaryWriter writer(out);
    harkio2::hark::HarkFileBinaryWriter::Helper helper(writer);

    const unsigned int nfft = 10;
    vector<vector<harkio2::Vector3<float>>> srcpos_m2pg(
            2, vector<harkio2::Vector3<float>>(
                3, harkio2::Vector3<float>(1, 2, 3)));
    vector<vector<vector<vector<C>>>> tf_m2pg(
        2, vector<vector<vector<C>>>(
            3, vector<vector<C>>(
                4, vector<C>(
                    nfft/2+1, C(10, 20)))));
    boost::shared_ptr<harkio2::hark::hgtf::M2PG1_0> m2pg10(
            new harkio2::hark::hgtf::M2PG1_0());
    m2pg10->SetND(2);
    m2pg10->SetNR(3);
    m2pg10->SetSourcePositions(srcpos_m2pg);
    m2pg10->SetTFs(tf_m2pg);
    helper.Write(*m2pg10);

    harkio2::hark::HarkFileBinaryReader reader(out);
    harkio2::hark::HarkFileBinaryReader::Helper rhelper(reader);

    auto rm2pg10 = rhelper.ReadM2PG1_0(4, nfft);
    BOOST_REQUIRE(rm2pg10);
    BOOST_CHECK_EQUAL(rm2pg10->GetNR(), 3);
    BOOST_REQUIRE_EQUAL(rm2pg10->GetSourcePositions().size(), 2);
    BOOST_REQUIRE_EQUAL(rm2pg10->GetSourcePositions()[0].size(), 3);
    BOOST_CHECK(rm2pg10->GetSourcePositions()[0][0] == harkio2::Vector3<float>(1, 2, 3));
    BOOST_REQUIRE_EQUAL(rm2pg10->GetTFs().size(), 2);
    BOOST_REQUIRE_EQUAL(rm2pg10->GetTFs()[0].size(), 3);
    BOOST_REQUIRE_EQUAL(rm2pg10->GetTFs()[0][0].size(), 4);
    BOOST_REQUIRE_EQUAL(rm2pg10->GetTFs()[0][0][0].size(), nfft/2+1);
    BOOST_CHECK_EQUAL(rm2pg10->GetTFs()[0][0][0][0], C(10, 20));
}

BOOST_AUTO_TEST_CASE(gtf10_test)
{
    typedef std::complex<float> C;

    std::stringstream out;

    harkio2::hark::HarkFileBinaryWriter writer(out);
    harkio2::hark::HarkFileBinaryWriter::Helper helper(writer);

    const unsigned int nmic = 3, ns = 4, nf = 5;
    vector<harkio2::Vector3<float>> srcpos_gtf(
            ns, harkio2::Vector3<float>(3, 2, 1));
    vector<unsigned int> table_gtf(nf, 7);
    vector<vector<vector<C>>> tf_gtf(
        ns, vector<vector<C>>(
            nmic, vector<C>(
                nf, C(10, 20))));
    boost::shared_ptr<harkio2::hark::hgtf::GTF1_0> gtf10(
            new harkio2::hark::hgtf::GTF1_0());
    gtf10->SetNSource(ns);
    gtf10->SetSourcePositions(srcpos_gtf);
    gtf10->SetNFIndex(nf);
    gtf10->SetFIndexToBinIndexTable(table_gtf);
    gtf10->SetTFs(tf_gtf);
    helper.Write(*gtf10);

    harkio2::hark::HarkFileBinaryReader reader(out);
    harkio2::hark::HarkFileBinaryReader::Helper rhelper(reader);

    auto rgtf10 = rhelper.ReadGTF1_0(nmic);
    BOOST_REQUIRE(rgtf10);
    BOOST_CHECK_EQUAL(rgtf10->GetNSource(), ns);
    BOOST_REQUIRE_EQUAL(rgtf10->GetSourcePositions().size(), ns);
    BOOST_CHECK_EQUAL(rgtf10->GetNFIndex(), nf);
    BOOST_REQUIRE_EQUAL(rgtf10->GetFIndexToBinIndexTable().size(), nf);
    BOOST_CHECK_EQUAL(rgtf10->GetFIndexToBinIndexTable()[0], 7);
    BOOST_REQUIRE_EQUAL(rgtf10->GetTFs().size(), ns);
    BOOST_REQUIRE_EQUAL(rgtf10->GetTFs()[0].size(), nmic);
    BOOST_REQUIRE_EQUAL(rgtf10->GetTFs()[0][0].size(), nf);
    BOOST_CHECK_EQUAL(rgtf10->GetTFs()[0][0][0], C(10, 20));
}

BOOST_AUTO_TEST_CASE(sm10_test)
{
    typedef std::complex<float> C;

    std::stringstream out;

    harkio2::hark::HarkFileBinaryWriter writer(out);
    harkio2::hark::HarkFileBinaryWriter::Helper helper(writer);

    const unsigned int nmic = 3, ns = 4, nf = 5;
    vector<harkio2::Vector3<float>> srcpos_sm(
            ns, harkio2::Vector3<float>(3, 2, 1));
    vector<unsigned int> table_sm(nf, 7);
    vector<vector<vector<C>>> sm_sm(
        ns, vector<vector<C>>(
            nmic, vector<C>(
                nf, C(10, 20))));
    boost::shared_ptr<harkio2::hark::hgtf::SM1_0> sm10(
            new harkio2::hark::hgtf::SM1_0());
    sm10->SetNSource(ns);
    sm10->SetSourcePositions(srcpos_sm);
    sm10->SetNFIndex(nf);
    sm10->SetFIndexToBinIndexTable(table_sm);
    sm10->SetSMs(sm_sm);
    helper.Write(*sm10);

    harkio2::hark::HarkFileBinaryReader reader(out);
    harkio2::hark::HarkFileBinaryReader::Helper rhelper(reader);

    auto rsm10 = rhelper.ReadSM1_0(nmic);
    BOOST_REQUIRE(rsm10);
    BOOST_CHECK_EQUAL(rsm10->GetNSource(), ns);
    BOOST_REQUIRE_EQUAL(rsm10->GetSourcePositions().size(), ns);
    BOOST_CHECK_EQUAL(rsm10->GetNFIndex(), nf);
    BOOST_REQUIRE_EQUAL(rsm10->GetFIndexToBinIndexTable().size(), nf);
    BOOST_CHECK_EQUAL(rsm10->GetFIndexToBinIndexTable()[0], 7);
    BOOST_REQUIRE_EQUAL(rsm10->GetSMs().size(), ns);
    BOOST_REQUIRE_EQUAL(rsm10->GetSMs()[0].size(), nmic);
    BOOST_REQUIRE_EQUAL(rsm10->GetSMs()[0][0].size(), nf);
    BOOST_CHECK_EQUAL(rsm10->GetSMs()[0][0][0], C(10, 20));
}

