#define BOOST_TEST_MODULE "testcase for xmlhelper functions"
#define BOOST_TEST_DYN_LINK
#include "common.hpp"

#include <boost/test/unit_test.hpp>

#include <iostream>
#include <glibmm/ustring.h>
#include <harkio2/xmlhelper/xmlhelper.hpp>

using namespace std;
using namespace Glib;
using namespace xmlpp;
namespace helper = harkio2::xmlhelper;

const char* xmlPosition1 =
R"xml(<?xml version="1.0" encoding="UTF-8"?>
<Position unit="none" type="unitvector">
    <X>1.0</X>
    <Y>1.0</Y>
    <Z>0.0</Z>
</Position>
)xml";

BOOST_AUTO_TEST_CASE(Contains_test)
{
    map<ustring, ustring> m;
    BOOST_CHECK_EQUAL(helper::Contains(m, "foo"), false);
    m["foo"] = "foo value";
    BOOST_CHECK_EQUAL(helper::Contains(m, "foo"), true);
}

BOOST_AUTO_TEST_CASE(GetLine_test)
{
    TextReader reader(reinterpret_cast<const unsigned char*>(xmlPosition1), strlen(xmlPosition1));

    BOOST_CHECK_EQUAL(reader.read(), true);
    BOOST_CHECK_EQUAL(reader.get_node_type(), TextReader::Element);
    BOOST_CHECK_EQUAL(reader.get_name(), "Position");
    BOOST_CHECK_EQUAL(helper::GetLine(reader), 2);

    BOOST_CHECK_EQUAL(reader.read(), true);
    BOOST_CHECK_EQUAL(reader.get_node_type(), TextReader::SignificantWhitespace);
    BOOST_CHECK_EQUAL(helper::GetLine(reader), 3);

    BOOST_CHECK_EQUAL(reader.read(), true);
    BOOST_CHECK_EQUAL(reader.get_node_type(), TextReader::Element);
    BOOST_CHECK_EQUAL(reader.get_name(), "X");
    BOOST_CHECK_EQUAL(helper::GetLine(reader), 3);

}

BOOST_AUTO_TEST_CASE(IsElement_test)
{
    TextReader reader(reinterpret_cast<const unsigned char*>(xmlPosition1), strlen(xmlPosition1));

    reader.read();
    BOOST_CHECK_EQUAL(reader.get_node_type(), TextReader::Element);
    BOOST_CHECK_EQUAL(reader.get_name(), "Position");
    BOOST_CHECK_EQUAL(helper::IsElement(reader, ustring("Position")), true);
    BOOST_CHECK_EQUAL(helper::IsElement(reader, "Position"), true);

    reader.read(); // significant whitespace
    reader.read(); // X
    reader.read(); // 1.0
    BOOST_CHECK_EQUAL(reader.get_value(), "1.0");

    reader.read(); // /X
    BOOST_CHECK_EQUAL(reader.get_node_type(), TextReader::EndElement);
    BOOST_CHECK_EQUAL(reader.get_name(), "X");
    BOOST_CHECK_EQUAL(helper::IsEndElement(reader, ustring("X")), true);
    BOOST_CHECK_EQUAL(helper::IsEndElement(reader, "X"), true);
}


